# jupyterlab_colab_bridge

JupyterLab Colab Bridge

[![Github Actions Status](https://github.com/signcl/jupyterlab-colab-bridge/workflows/Build/badge.svg)](https://github.com/signcl/jupyterlab-colab-bridge/actions/workflows/build.yml)

A JupyterLab extension to support Colab form syntax.

<img width="1481" alt="CleanShot 2022-10-05 at 10 42 11@2x" src="https://user-images.githubusercontent.com/2572987/194103256-3ca8ec26-d97c-4f84-b7c2-509d38443089.png">

## Requirements

- JupyterLab >= 3.0

## Install

To install the extension, execute:

```bash
pip install jupyterlab_colab_bridge
```

## Uninstall

To remove the extension, execute:

```bash
pip uninstall jupyterlab_colab_bridge
```

## Usage

Supported Colab commands:

### Title

Display in title style:

```
#@title This is a title
```

Title can only be set on the first non-empty line. It supports markdown.

You can set `display-mode` on it:

```
#@title ##This is a title {display-mode: 'code'}
```

There're three modes supported: `code`, `form`, `both`.

Mode can also be switched from the cell toolbar buttons.

### Markdown

Parse content using Markdown format:

```
#@markdown ## title
```

Multiple lines will be parsed together:

````
#@markdown ```
#@markdown a = 1
#@markdown ```
````

### Param

Should be added after a variable assignment:

```
a = 123 #@param
```

By default, the `type` is `string`.

You can set the type explicitly:

```
a = 123 #@param {type: "number"}
a = 10 #@param {type: "integer"}
a = True #@param {type: "boolean"}
a = b #@param {type: "raw"}
a = '2018-03-22' #@param {type:"date"}
a = 10 #@param {type: "slider"}
a = 10 #@param {type: "slider", min: 0, max: 100, step: 1}
```

You can provide a list of values to choose from:

```
a = 123 #@param {type: "number"} [123, 3, 4]
a = "name" #@param {type: "string"} ["name", "value", "foo"]
a = "name" #@param {type: "raw"} ["name", True, 123]
```

You can also combine the value list and the input:

```
a = "name" #@param {type: "string", allow-input: true} ["name", "value", "foo"]
```

## Contributing

### Use conda environment

It's highly recommended to use the conda environment instead of installing jupyterlab on your local environment. It's easier to start a fresh new environment when something goes wrong.

Follow the official [guide](https://jupyterlab.readthedocs.io/en/stable/extension/extension_tutorial.html#install-nodejs-jupyterlab-etc-in-a-conda-environment) to install conda and start a new environment.

### Development install

Note: You will need NodeJS to build the extension package.

The `jlpm` command is JupyterLab's pinned version of
[yarn](https://yarnpkg.com/) that is installed with JupyterLab. You may use
`yarn` or `npm` in lieu of `jlpm` below.

```bash
# Clone the repo to your local environment
# Change directory to the jupyterlab_colab_bridge directory
# Install package in development mode
pip install -e .
# Link your development version of the extension with JupyterLab
jupyter labextension develop . --overwrite
# Rebuild extension Typescript source after making changes
jlpm build
```

You can watch the source directory and run JupyterLab at the same time in different terminals to watch for changes in the extension's source and automatically rebuild the extension.

```bash
# Watch the source directory in one terminal, automatically rebuilding when needed
jlpm watch
# Run JupyterLab in another terminal
jupyter lab
```

With the watch command running, every saved change will immediately be built locally and available in your running JupyterLab. Refresh JupyterLab to load the change in your browser (you may need to wait several seconds for the extension to be rebuilt).

By default, the `jlpm build` command generates the source maps for this extension to make it easier to debug using the browser dev tools. To also generate source maps for the JupyterLab core extensions, you can run the following command:

```bash
jupyter lab build --minimize=False
```

### Development uninstall

```bash
pip uninstall jupyterlab_colab_bridge
```

In development mode, you will also need to remove the symlink created by `jupyter labextension develop`
command. To find its location, you can run `jupyter labextension list` to figure out where the `labextensions`
folder is located. Then you can remove the symlink named `jupyterlab-colab-bridge` within that folder.

### Testing the extension

#### Frontend tests

This extension is using [Jest](https://jestjs.io/) for JavaScript code testing.

To execute them, execute:

```sh
jlpm
jlpm test
```

#### Integration tests

This extension uses [Playwright](https://playwright.dev/docs/intro/) for the integration tests (aka user level tests).
More precisely, the JupyterLab helper [Galata](https://github.com/jupyterlab/jupyterlab/tree/master/galata) is used to handle testing the extension in JupyterLab.

More information are provided within the [ui-tests](./ui-tests/README.md) README.

### Packaging the extension

See [RELEASE](RELEASE.md)

### i18n

You can check the `LocaleTextName` in `src/constants.ts`.

To support a new language, add a new translation file under `src/locale`. Populate all translated text there. Then update the `localeTextMap` and `localeTextFnMap` in `src/locale/index.ts`.

For now, the language is set to `zh_CN`. If you want to make it dynamic, use `Navigator.language` API provided by browsers to get the user language and set it in `colabCommand/index.ts`.
