import { strToJSON } from '../utils/utils';
import { CommandType, QuoteType } from '../constants';

export const colabCommandTrimmer = {
  [CommandType.Title]: (line: string): { value: string } => {
    return {
      value: [
        ...line.replace(/#\s*@title/, '#@title').matchAll(/#@title(.*)/g)
      ][0][1].trim()
    };
  },
  [CommandType.Param]: (
    line: string
  ): {
    name: string | undefined;
    value: string | number | undefined;
    rawValue: string | number | undefined;
    config: any | undefined;
    valueList: any[] | undefined;
    quoteType: QuoteType | undefined;
  } => {
    let [code, colabCommand] = line
      .replace(/#\s*@param/, '#@param')
      .split('#@param');
    code = code.trim();
    colabCommand = colabCommand.trim();

    // extract the config
    const configStr = [...colabCommand.matchAll(/{.*?}/g)][0]?.pop();
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const config =
      configStr &&
      JSON.parse(strToJSON(configStr.replace('allow-input', 'allowInput')));

    // extract the dropdown list
    const valueListStr = [...colabCommand.matchAll(/\[.*\]/g)][0]?.pop();
    let valueList;
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
    if (config?.type === 'raw') {
      valueList =
        valueListStr &&
        valueListStr
          .slice(1, valueListStr.length - 1)
          .split(',')
          .map(value => value.trim());
    } else {
      // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
      valueList = valueListStr && JSON.parse(strToJSON(valueListStr));
    }

    // extract the variable name
    let name, value, rawValue;
    // eslint-disable-next-line prefer-const
    [, name, rawValue] = [...code.matchAll(/\s*(.*?)\s*=\s*(.*)\s*/g)][0] || [];
    if ([...code.matchAll(/\s*(.*?)\s*=\s*["'](.*)["']\s*/g)][0]) {
      [, , value] = [...code.matchAll(/\s*(.*?)\s*=\s*["'](.*)["']\s*/g)][0];
    } else {
      [, , value] = [...code.matchAll(/\s*(.*?)\s*=\s*(.*)\s*/g)][0] || [];
    }

    let quoteType;
    if (/'.*'/.exec(rawValue ?? '')) {
      quoteType = QuoteType.Single;
    } else if (/".*"/.exec(rawValue ?? '')) {
      quoteType = QuoteType.Double;
    }
    if (!(/'.*'/.exec(rawValue ?? '') || /".*"/.exec(rawValue ?? ''))) {
      value = rawValue;
    }

    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    return { name, value, rawValue, config, valueList, quoteType };
  },
  [CommandType.Markdown]: (line: string): string => {
    return [
      ...line.replace(/#\s*@markdown/, '#@markdown').matchAll(/#@markdown(.*)/g)
    ][0][1].trim();
  }
};
