import { CommandType, QuoteType } from '../constants';

const getQuote = (type: QuoteType | undefined) => {
  if (type === QuoteType.Single) {
    return "'";
  } else {
    return '"';
  }
};

const escapeString = (str: string, type: QuoteType | undefined) => {
  if (type === QuoteType.Single) {
    str = str.replaceAll("'", "\\'");
  } else {
    str = str.replaceAll('"', '\\"');
  }
  return str;
};

export const colabCommandUpdater: Record<
  CommandType.Param,
  (
    line: string,
    replaceStr: string | undefined,
    withStr: string,
    quoteType: QuoteType | undefined,
    needQuotation?: boolean
  ) => string
> = {
  [CommandType.Param]: (
    line: string,
    replaceStr: string | undefined,
    withStr: string,
    quoteType: QuoteType | undefined,
    needQuotation?: boolean
  ): string => {
    if (replaceStr) {
      return line.replace(
        // handle the normal situation like `aa = 123` or `aa = '123`
        RegExp(String.raw`=(.*)${replaceStr}(.*)#(\s*)@param`, 'g'),
        `=$1${withStr}$2#$3@param`
      );
    } else {
      // handle the situation like `aa = ` or `aa = ''`
      return line.replace(
        RegExp(String.raw`=.*#(\s*)@param`, 'g'),
        `= ${
          needQuotation
            ? getQuote(quoteType) +
              escapeString(withStr, quoteType) +
              getQuote(quoteType)
            : withStr
        } #$1@param`
      );
    }
  }
};
