import {
  JupyterFrontEnd,
  JupyterFrontEndPlugin
} from '@jupyterlab/application';
import {
  INotebookTracker,
  NotebookPanel,
  StaticNotebook
} from '@jupyterlab/notebook';
import { IEditorServices } from '@jupyterlab/codeeditor';
import { CodeCell } from '@jupyterlab/cells';
import { ColabCodeCell } from './colabCell';

class ColabCodeContentFactory extends NotebookPanel.ContentFactory {
  /**
   * Create a new markdown cell widget.
   *
   * #### Notes
   * If no cell content factory is passed in with the options, the one on the
   * notebook content factory is used.
   */
  createCodeCell(options: CodeCell.IOptions, parent: StaticNotebook): CodeCell {
    if (!options.contentFactory) {
      options.contentFactory = this;
    }
    return new ColabCodeCell(options).initializeState();
  }
}

/**
 * The notebook cell factory provider.
 */
const codeFactory: JupyterFrontEndPlugin<NotebookPanel.IContentFactory> = {
  id: '@signcl/jupyterlab-colab-bridge:factory',
  provides: NotebookPanel.IContentFactory,
  requires: [IEditorServices],
  autoStart: true,
  activate: (app: JupyterFrontEnd, editorServices: IEditorServices) => {
    console.log('Using jupyterlab-colab-bridge:factory');
    // eslint-disable-next-line @typescript-eslint/unbound-method
    const editorFactory = editorServices.factoryService.newInlineEditor;
    return new ColabCodeContentFactory({ editorFactory });
  }
};

/**
 * The notebook cell executor.
 */
const executor: JupyterFrontEndPlugin<void> = {
  id: '@signcl/jupyterlab-colab-bridge:executor',
  requires: [INotebookTracker],
  autoStart: true,
  activate: (app: JupyterFrontEnd, tracker: INotebookTracker) => {
    console.log('Using jupyterlab-colab-bridge:executor');
    tracker.activeCellChanged.connect(event => {
      const cell = event.activeCell;
      if (
        cell &&
        cell.model.type === 'code' &&
        /#\s*@(title|param|markdown)/.exec(cell.model.value.text)
      ) {
        setTimeout(() => (cell as ColabCodeCell).injectToolbarBtns(), 0);
      }
    });
    return;
  }
};

/**
 * Export the plugins as default.
 */
const plugins: JupyterFrontEndPlugin<any>[] = [codeFactory, executor];
export default plugins;
