import { localeText } from './index';
import { LocaleTextName } from '../constants';

describe('test zh_CN locale', () => {
  test('should set the lang as zh_CN', () => {
    localeText.setLang('zh_CN');
    expect(localeText.getLang()).toBe('zh_CN');
  });

  test('should return zh_CN locale text', () => {
    expect(localeText.getText(LocaleTextName.INTEGER_ERROR_MESSAGE)).toBe(
      '不是合法的整数'
    );
  });

  test('should return zh_CN locale fn text', () => {
    expect(
      localeText.getTextFn(LocaleTextName.ASSERT_ERROR_MESSAGE_FN)('name')
    ).toBe('无法找到[变量名]，请检查代码格式');
  });

  test('should return zh_CN locale fn text for not matched field', () => {
    expect(
      localeText.getTextFn(LocaleTextName.ASSERT_ERROR_MESSAGE_FN)('123')
    ).toBe('无法找到[123]，请检查代码格式');
  });

  test('should return zh_CN locale fn text', () => {
    expect(
      localeText.getTextFn(LocaleTextName.CELL_BUILD_ERROR_MESSAGE_FN)('123')
    ).toBe('解析出错，123 不是正确的命令格式');
  });
});

describe('test en locale', () => {
  test('should set the lang as en', () => {
    localeText.setLang('en');
    expect(localeText.getLang()).toBe('en');
  });

  test('should return en locale text', () => {
    expect(localeText.getText(LocaleTextName.INTEGER_ERROR_MESSAGE)).toBe(
      'illegal integer'
    );
  });

  test('should return en locale fn text', () => {
    expect(
      localeText.getTextFn(LocaleTextName.CELL_BUILD_ERROR_MESSAGE_FN)('123')
    ).toBe('parse error,123 is not the correct command format');
  });

  test('should return en locale fn text for not matched field', () => {
    expect(
      localeText.getTextFn(LocaleTextName.ASSERT_ERROR_MESSAGE_FN)('123')
    ).toBe('could not find[123], please check the code');
  });

  test('should return en locale fn text', () => {
    expect(
      localeText.getTextFn(LocaleTextName.ASSERT_ERROR_MESSAGE_FN)('name')
    ).toBe('could not find[variable name], please check the code');
  });
});

describe('test default locale', () => {
  test('should set the lang as en if the translations are not available', () => {
    localeText.setLang('abc');
    expect(localeText.getLang()).toBe('en');
  });

  test('should return en locale text', () => {
    expect(localeText.getText(LocaleTextName.INTEGER_ERROR_MESSAGE)).toBe(
      'illegal integer'
    );
  });

  test('should return zh_CN locale fn text', () => {
    expect(
      localeText.getTextFn(LocaleTextName.ASSERT_ERROR_MESSAGE_FN)('name')
    ).toBe('could not find[variable name], please check the code');
  });

  test('should return en locale fn text for not matched field', () => {
    expect(
      localeText.getTextFn(LocaleTextName.ASSERT_ERROR_MESSAGE_FN)('123')
    ).toBe('could not find[123], please check the code');
  });

  test('should return zh_CN locale fn text', () => {
    expect(
      localeText.getTextFn(LocaleTextName.CELL_BUILD_ERROR_MESSAGE_FN)('123')
    ).toBe('parse error,123 is not the correct command format');
  });
});
