import { LRUCache } from './cache';

const cache = new LRUCache(3);
const cache2 = new LRUCache();

describe('test LRUCache', () => {
  afterEach(() => {
    cache.clear();
  });

  test('should cache the value', () => {
    cache.set('1', '11');
    expect(cache.get('1')).toBe('11');
    cache.set('2', '22');
    expect(cache.get('2')).toBe('22');
  });

  test('should return undefined for not cached key', () => {
    expect(cache.get('1')).toBeFalsy();
  });

  test('should remove the least used cache', () => {
    cache.set('1', '11');
    cache.set('2', '22');
    cache.set('3', '33');
    cache.get('1');
    cache.get('1');
    cache.get('3');
    cache.set('4', '44');
    expect(cache.get('4')).toBe('44');
    expect(cache.get('2')).toBeFalsy();
  });

  test('should clear the whole cache', () => {
    cache.set('1', '11');
    cache.set('2', '22');
    cache.set('3', '33');
    cache.clear();
    expect(cache.get('1')).toBeFalsy();
    expect(cache.get('2')).toBeFalsy();
    expect(cache.get('3')).toBeFalsy();
  });

  test('should work with defualt maxSize', () => {
    cache2.set('1', '11');
    cache2.set('2', '22');
    cache2.set('3', '33');
    cache2.clear();
    expect(cache2.get('1')).toBeFalsy();
    expect(cache2.get('2')).toBeFalsy();
    expect(cache2.get('3')).toBeFalsy();
  });
});
