/* eslint-disable @typescript-eslint/no-unsafe-assignment */
// a simple LRU cache implementation
export class LRUCache {
  constructor(maxSize = 100) {
    this.maxSize = maxSize;
    this.storage = {};
    this.hitCount = {};
  }

  private maxSize: number;
  private storage: Record<string, any>;
  private hitCount: Record<string, number>;

  set(key: string, value: any): void {
    if (Object.keys(this.storage).length >= this.maxSize) {
      const keys = Object.keys(this.hitCount);
      keys.sort((a, b) => this.hitCount[a] - this.hitCount[b]);
      delete this.storage[keys[0]];
      delete this.hitCount[keys[0]];
    }
    this.storage[key] = value;
    this.hitCount[key] = 0;
  }

  get(key: string): any | undefined {
    if (this.storage[key]) {
      this.hitCount[key]++;
      return this.storage[key];
    }
  }

  remove(key: string): void {
    if (this.storage[key]) {
      delete this.storage[key];
      delete this.hitCount[key];
    }
  }

  clear(): void {
    this.storage = {};
    this.hitCount = {};
  }
}
