/* eslint-disable @typescript-eslint/no-unsafe-return */
/* eslint-disable @typescript-eslint/ban-types */
/* eslint-disable @typescript-eslint/naming-convention */

interface Subscription {
  [key: number]: Function;
}

interface Subscriptions {
  [key: string]: Subscription;
}

class PubSub {
  constructor() {
    this.subIds = 0;
    this.subscriptions = {};
  }

  private subIds: number;
  private subscriptions: Subscriptions;

  subscribe(topic: string, fn: Function) {
    if (!this.subscriptions[topic]) {
      this.subscriptions[topic] = {};
    }
    const token = ++this.subIds;
    // Validate topic name and function constructor
    this.subscriptions[topic][token] = fn;

    return () => this.unsubscribe(topic, token);
  }

  publish(topic: string, ...args: any) {
    const subs = this.subscriptions[topic];
    if (!subs) {
      return false;
    }
    // eslint-disable-next-line @typescript-eslint/no-unsafe-call
    Object.values(subs).forEach(sub => sub(...args));
  }

  unsubscribe(topic: string, token?: number) {
    if (!token) {
      delete this.subscriptions[topic]; // Delete all subscriptions for the topic
    } else {
      this.subscriptions[topic] && delete this.subscriptions[topic][token]; // Delete specific subscription
    }
  }
}

export default new PubSub();
